#include <hpgcc49.h>
#include <hpgraphics.h>

#define WINDOW_WIDTH  				131
#define WINDOW_HEIGHT  				80
#define NULL 						0
#define PI         					((float) 3.141592654)
#define DEG_TO_RAD(ang) 			((ang) * PI / 180.0)
#define EPSILON_E5 					(float) (1E-5)
#define pi                          PI
#define MAX_FUNCTION				42

#define COL_H						6

#define COL_U1						COL_H * 0
#define COL_U2						COL_H * 1
#define COL_W3						COL_H * 2
#define COL_W4						COL_H * 3
#define COL_STEPU					COL_H * 4
#define COL_STEPW					COL_H * 5

#define COL_OPEN					COL_H * 6
#define COL_A						COL_H * 7
#define COL_B 						COL_H * 8
#define COL_C						COL_H * 9

#define INC 						0.05

//																								typedefs
typedef struct {
	float x,y,z;
} VECTOR3D, POINT3D, *VECTOR3D_PTR, *POINT3D_PTR;

typedef struct {
	POINT3D p0; 									
	VECTOR3D n; 									
} PLANE, *PLANE_PTR;

typedef struct {
	union {
		float M[4][4];
		struct {
			float M00, M01, M02, M03;
			float M10, M11, M12, M13;
			float M20, M21, M22, M23;
			float M30, M31, M32, M33;
		};
	};
} MATRICE, *MATRICE_PTR;

typedef struct {
	union {
		float M[4]; 
		struct { float x,y,z,w;};
	};
} VECTOR4D, POINT4D, *VECTOR4D_PTR, *POINT4D_PTR;

typedef struct {
	union {     
		struct {float x,y,z,w;};    
		struct  {POINT4D  v;};             
	}; 
} VERTEX, *VERTEX_PTR;

typedef struct {					 
	int cull;;
	int vert[4];
} POLYGONE, *POLYGONE_PTR;

typedef struct {   
	POINT4D world_pos;    
	VERTEX_PTR vlist;
	POLYGONE_PTR plist;  
	POLYGONE_PTR *pplist;  
} OBJECT, *OBJECT_PTR;

typedef struct {   					
	POINT4D pos;    								
	VECTOR4D dir;  	 							
	float view_dist;  								
	float fov;        							
	float near_z;    							
	float far_z;      					
	PLANE rt_plane;  					
	PLANE lt_plane;  						
	PLANE tp_plane;  						
	PLANE bt_plane;  						              
	float viewplane_width;     					
	float viewplane_height;  					  	
	float viewport_width; 						   
	float viewport_height;
	float viewport_center_x;  						
	float viewport_center_y;
	float aspect_ratio;
	MATRICE mcam;   								
	MATRICE mper; 							    
	MATRICE mscr; 							    
} CAMERA, *CAMERA_PTR;

typedef struct {
	VECTOR3D scale;
	float u_min,u_max;
	float v_min,v_max;	
	int open;
	int stepu;
	int stepw;
	void (*compute_surface)();
} SURFACE;

typedef struct {
	char *name;
	char *x;
	char *y;
	char *z;
} SURFACE_INFO;

typedef struct {
	float a;
	float b;
	float c;
	int flag_a;
	int flag_b;
	int flag_c;
} VARIABLE;

//																								constants, macros, prototypes
const MATRICE ID_MAT = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};

#define MAT_IDENTITY(m) {memcpy((void *)(m), (void *)&ID_MAT, sizeof(MATRICE));}
#define MAT_COPY(src_mat, dest_mat) {memcpy((void *)(dest_mat), (void *)(src_mat), sizeof(MATRICE) ); }
#define SWAP(a,b,t) {t=a; a=b; b=t;}

void compute_vertex();
int compare_polygone(const void *arg1, const void *arg2);
void init_plane(PLANE_PTR plane, POINT3D_PTR p0, VECTOR3D_PTR normal);
void init_matrice(MATRICE_PTR ma, 
				float m00, float m01, float m02, float m03, float m10, float m11, float m12, float m13,
				float m20, float m21, float m22, float m23, float m30, float m31, float m32, float m33);				
void multiply_matrice(MATRICE_PTR ma, MATRICE_PTR mb, MATRICE_PTR mprod);
void build_rotation_matrice(float theta_x, float theta_y, float theta_z, MATRICE_PTR mrot);
void init_camera(CAMERA_PTR cam, POINT4D_PTR cam_pos, VECTOR4D_PTR cam_dir, float near_z, float far_z, 
					float fov, float viewport_width,  float viewport_height);

void compute_plane(), 				compute_helicoid(), 	compute_paraboloid(), 		compute_funnel(),			compute_tractroid(), 	compute_hyperboloid();	
void compute_swallow(), 			compute_wallis(), 		compute_spherical_wave(), 	compute_bow_tie(),			compute_trash_can(), 	compute_spring();
void compute_cylinder(), 			compute_disk(), 		compute_wave_cosine(), 		compute_flying_saucer(),	compute_cornucopia(), 	compute_enneper(); 		
void compute_handkerchief(), 		compute_klein_bottle(), compute_monkey(), 			compute_plucker(),			compute_roman(), 		compute_shoe();	
void compute_screw(),		 		compute_whitney(), 		compute_catenoid(), 		compute_snail(), 			compute_cosine(),		compute_ellipsoid(); 	
void compute_eight(), 				compute_kidney(), 		compute_drop(), 			compute_pillow(), 			compute_fish(), 		compute_horn();
void compute_pseudo_cross_cap(), 	compute_cresent(), 		compute_apple(), 			compute_trefoilknot(), 		compute_torus(), 		compute_antisymetric_torus();
void compute_turn_torus();

//																								globals			
CAMERA cam; POINT4D cam_pos = { 0, 0, 0, 1 }; VECTOR4D cam_dir = { 0, 0, 0, 1 };
OBJECT surface; 
int raster_x[4], raster_y[4];
MATRICE mrot; float x_angle = 10.0; float y_angle = 0.0; float z_angle = 0.0;
int polygone, vertex; POINT4D presult; int col,row; float sum;
int num_verts, num_u, num_polys;
int surface_index;
float u_inc,v_inc;
int display = 0;
int info = 0;
VECTOR3D *vertex_array;
VARIABLE variable;
char buffer[50];

//			scale				 u range	  		  v range			 open		stepu	stepw	function
SURFACE surface_array[] = {
	{{  1.0,  1.0,  1.0},   	-12.0, 12.0, 		-12.0, 12.0, 		 	1,		10,		10,		compute_plane},
	{{  1.5,  1.5,  1.5},   	-2.0, 2.0, 			0.0, 2.0*pi, 		 	1,		7,		11,		compute_helicoid},
	{{  0.5,  0.5,  0.5}, 		-pi,pi,			 	0.0, 2.0, 		 		0,		10,		10,		compute_paraboloid},
	{{  3.0,  3.0,  3.0},    	0.1, 3.5, 			0.0, 2.5*pi, 		 	1,		10,		10,		compute_funnel},
	{{  6.5,  6.5,  6.5},		-0.6,3.4, 		   	0.0,2.5*pi, 			1, 		10,		10,		compute_tractroid},
	{{  1.5,  1.5,  1.5},		-2.8,3.4,			-1.8, 2.2, 			 	0,		10,		10,		compute_hyperboloid},
	{{  3.0,  3.0,  3.0},  		-2.0, 2.0, 			-0.75, 0.75, 		 	1,		10,		10,		compute_swallow},
	{{ 10.0, 10.0, 10.0},   	-1.0, 1.0, 			0.0, 1.0,	 		 	1,		10,		10,		compute_wallis},
	{{  0.8,  0.8,  0.8},   	0.0, 14.0, 			0.0, 6.28,	 		 	1,		10,		10,		compute_spherical_wave},	
	{{  4.0,  4.0,  4.0},   	-3.1,3.65, 			-1.8,5.0,	 		 	1,		14,		14,		compute_bow_tie},
	{{  3.0,  3.0,  3.0},   	0.0,2.0*pi, 		0.0,2.0,	 		 	0,		14,		14,		compute_trash_can},
	{{  4.0,  4.0,  4.0},   	-8.65,6.3, 			0.25,6.55,	 		 	1,		14,		14,		compute_spring},
	{{  4.0,  4.0,  4.0},   	-pi,pi, 			0.0,3.0,	 		 	0,		10,		10,		compute_cylinder},
	{{  3.0,  3.0,  3.0},   	0.0,2.0*pi, 		0.0,2.0*pi,	 		 	0,		14,		14,		compute_disk},
	{{  1.0,  1.0,  1.0},   	-12.0, 12.0, 		-12.0, 12.0,		 	1,		14,		14,		compute_wave_cosine},
	{{  8.0,  8.0,  8.0},   	-pi,pi, 			0.0,3.40,				1,		14,		14,		compute_flying_saucer},
	{{ 0.02, 0.02, 0.02}, 		-pi, pi, 			0.0, 2*pi, 			 	0,		15,		15,		compute_cornucopia},		
	{{  1.5,  1.5,  1.5},      -2.0, 2.0, 			-2.0, 2.0, 			 	0,		10,		10,		compute_enneper},
	{{  5.0,  5.0,  5.0},      -1.0, 1.0, 			-1.0, 1.0,			 	1,		10,		10,		compute_handkerchief},
	{{  0.6,  0.6,  0.6},     	0.1,2.4*pi, 		0.0, 2.3*pi, 		 	0,		13,		13,		compute_klein_bottle},
	{{  5.0,  5.0,  5.0},     	-0.7, 1.3, 			-1.0, 1.0, 			 	1,		10,		10,		compute_monkey},
	{{  6.0,  6.0,  6.0},     	-2.0, 2.0, 			-1.0, 1.0, 			 	1,		10,		10,		compute_plucker},
	{{ 10.0, 10.0, 10.0},   	0.0, 1.0, 			0.0, 2.5*pi, 		 	1,		10,		10,		compute_roman},
	{{  3.0,  3.0,  3.0},     	-2.0, 2.0, 			-2.0, 2.0, 			 	1,		10,		10,		compute_shoe},
	{{  2.0,  2.0,  2.0},     	-4.0, 4.0, 			0.0, 2.0*pi, 		 	1,		10,		10,		compute_screw},
	{{  5.0,  5.0,  5.0},     	-1.5, 1.5, 			-1.5, 1.5, 			 	1,		10,		10,		compute_whitney},
	{{  5.0,  5.0,  5.0},     	0.0,2.5*pi, 		-1.0, 1.0, 			 	1,		10,		10,		compute_catenoid},
	{{  1.5,  1.5,  1.5},		0.0, 2.5*pi, 		-1.5*pi, 1.5*pi, 		0, 		15,		15,		compute_snail},					
	{{  8.0,  8.0,  8.0},		-pi,pi, 			-pi,pi, 				0, 		15,		15,		compute_cosine},				
	{{ 10.0, 10.0, 10.0},		0.0,2.0*pi, 		0.0,3.0*pi, 			0, 		15,		15,		compute_ellipsoid},				
	{{  9.0,  9.0,  9.0},		0.0,2.0*pi, 		-pi/1.8,pi/1.8, 		0, 		10,		10,		compute_eight},					
	{{  3.0,  3.0,  3.0},		0.0,2.0*pi, 		-pi/1.8,pi/1.8, 		0, 		10,		10,		compute_kidney},				
	{{  8.0,  8.0,  8.0},		0.0,1.1*pi,	 		0.0,2.5*pi,	 			0, 		10,		10,		compute_drop},					
	{{  8.0,  8.0,  8.0},		0.0,1.11*pi,	 	-1.15*pi, 1.15*pi,		1, 		15,		15,		compute_pillow},				
	{{ 12.0, 12.0, 12.0},		0.0,1.11*pi,	 	0.0,2.2*pi,				1, 		10,		10,		compute_fish},			
	{{  4.0,  4.0,  4.0},		0.0,1.0,		 	0.7,7.61	,			1, 		10,		10,		compute_horn},					
	{{ 11.0, 11.0, 11.0},   	-1.0,1.0, 			-1.02*pi, 1.02*pi, 		0,		10,		10,		compute_pseudo_cross_cap},		
	{{  3.0,  3.0,  3.0},   	0.0,1.0, 			0.0,1.0, 				0,		10,		10,		compute_cresent},				
	{{  1.3,  1.3,  1.3},   	-1.9,4.38, 			-pi, pi, 				0,		13,		13,		compute_apple},					
	{{  2.0,  2.0,  2.0},   	-2.0*pi,2.0*pi, 	-pi, pi, 				0,		15,		15,		compute_trefoilknot},			
	{{  3.0,  3.0,  3.0},   	0.0,2.0*pi,		 	0.4,2.4*pi, 			0,		13,		13,		compute_torus},					
	{{  3.0,  3.0,  3.0},   	0.0,2.0*pi,		 	0.4,2.4*pi, 			0,		13,		13,		compute_antisymetric_torus},	
	{{  3.0,  3.0,  3.0},   	0.3,2.3*pi,		 	0.4,2.4*pi, 			0,		13,		13,		compute_turn_torus}};

SURFACE_INFO surface_info[] = {		
	{"0:Plane",   
							"x=  u ",
							"y=  0.0 ",
							"z=  w " },

	{"1:Helicoid",   
							"x=  u cos(w)",
							"y=  u sin(w)",
							"z=  a w" },
							
	{"2:Paraboloid",   
							"x=  a w cos(u)",
							"y=  a w sin(u)",
							"z=  b w^2" },
							

	{"3:Funnel",   
							"x=  u cos(w)",
							"y=  u sin(w)",
							"z=  log(u)" },

	{"4:Tractroid",   
							"x=  sech(u) cos(w)",
							"y=  sech(u) sin(w)",
							"z=  u - tanh(u)" },
								
	{"5:Hyperboloid",   
							"x= a cosh(w) cos(u)",
							"y= a cosh(w) sin(u)",
							"z= b sinh(w)" },
	
	{"6:Swallow Surface",   
							"x= u w^2 + 3 w^4",
							"y= -2 u w - 4 w^3",
							"z= u" },
							
		
	{"7:Walliss Conical Edge",   
							"x= w cos(u)",
							"y= w sin(u)",
							"z= c sqrt(a^2 - b^2 cos^2(u))" },

	{"8:Spherical Wave",   
							"x= u cos(cos(u)) cos(w)",
							"y= u cos(cos(u)) sin(w)",
							"z= u sin(cos(u))" },

	{"9:Bow Tie",   
							"x= sin(w) cos(u)",
							"y= 2 cos(w)",
							"z= 4 sin(w) cos(w)" },

	{"10:Trash Can",   
							"x= (b + w) cos(u)",
							"y= w sin(u)",
							"z= a w^2" },
	
	{"11:Spring",   
							"x= (a + b cos(w)) cos(u)",
							"y= (a + b cos(w)) sin(u)",
							"z= b (sin(w) + c u/pi)" },

	{"12:Cylinder",   
							"x= a cos(u)",
							"y= a sin(u)",
							"z= w" },

	{"13:Disk",   
							"x= sin(u) / (a + sin(w))",
							"y= cos(w) / b",
							"z= cos(u) / (a + sin(w))" },

	{"14:Wave Cosine",   
							"x= u",
							"y= a cos(b sqrt(u^2 + w^2))",
							"z= w" },

	{"15:Flying Saucer",   
							"x= cos(u) cos(w)",
							"y= sin^3(u)",
							"z= cos(u) sin(w)" },

	{"16:Cornucopia",   
							"x= e^(b w) cos(w) +\n   e^(a w) cos(u) cos(w)",
							"y= e^(b w) sin(w) +\n   e^(a w) cos(u) sin(w)",
							"z= e^(a w) sin(u)" },
	
	{"17:Ennepers Surface",   
							"x= u - (u^3/3) + u w^2)",
							"y= w - (w^3/3) + u^2 w)",
							"z= u^3/3 + u w^2 + 2(u^2 - w^2)" },

	{"18:Handkerchief Surface",   
							"x=  u ",
							"y=  v ",
							"z=  u^2 - v^2" },


	{"19:Klein Bottle",   		"equation too long","",""},	
	
							/*
								0 <= u < pi
							x= a cos(u) (1 + sin(u)) + (4 (1-cos(u) /2)) cos(u)cos(w)
							y= b sin(u) + (4(1-cos(u)/2)) sin(u) cos(w)"
							z= (4 (1 - cos(u)/2)) sin(w) 
							
								pi < u <= 2pi
							x= a cos(u) (1 + sin(u)) + (4 (1 - cos(u)/2)) cos(w + pi),
							y= b sin(u) 
							z= (4 (1 - cos(u)/2)) sin(w)" }
							*/

	{"20:Monkey Saddle",   
							"x=  u ",
							"y=  w ",
							"z=  u^3 - 3 u w^2" },

	{"21:Plckers Conoid",   
							"x=  u sqrt(1 - w^2) ",
							"y=  u w ",
							"z=  1 - w^2" },

	{"22:Roman Surface",   
							"x= 2 u cos(w) sqrt(1 - u^2) ",
							"y= 2 u sin(w) sqrt(1 - u^2)",
							"z= 1 - 2 u^2 cos(w) cos(w)" },

	{"23:Shoe Surface",   
							"x= u ",
							"y= w ",
							"z= u^3/3 - w^2/2" },

	{"24:Steinbach Screw",   
							"x= u cos(w) ",
							"y= u sin(w) ",
							"z= w cos(u) " },
							
	{"25:Whitney Umbrella",   
							"x= u w",
							"y= u  ",
							"z= w^2 " },
							
	{"26:Catenoid",   
							"x= a cosh(w/a) cos(u)",
							"y= a cosh(w/a) sin(u)",
							"z= w " },
		
	{"27:Snail Surface",   
							"x=  u cos(w) sin(u)",
							"y=  u cos(u) cos(w)",
							"z=  -u sin(w)" },
	
	{"28:Cosine Surface",  
							"x=  cos(u)",
							"y=  cos(w)", 
							"z=  cos(u + w)" },

	{"29:Ellipsoid",  
							"x=  a cos(u) sin(w)",
							"y=  b sin(u) sin(w)",
							"z=  c cos(w)" },

	{"30:Eight Surface",  
							"x=  cos(u) sin(2 w)",
							"y=  sin(u) sin(2 w)",
							"z=  sin(w)" },

	{"31:Kidney Surface",  
							"x=  cos(u) (3 cos(w) - cos(3 w))",
							"y=  sin(u) (3 cos(w) - cos(3 w))",
							"z=  3 sin(w) - sin(3 w)" },
	
	{"32:Drop Surface",  
							"x=  a (b - cos(u)) sin(u) cos(w)",
							"y=  a (b - cos(u)) sin(u) sin(w)",
							"z=  cos(u)" },
													
	{"33:Pillow Shape",  
							"x=  cos(u)",
							"y=  cos(w)",
							"z=  a sin(u) sin(v)" },
							
	{"34:Fish Surface",  
							"x= (cos(u) - cos(2 u)) cos(w)/4",
							"y= (sin(u) - sin(2 u)) sin(w)/4",
							"z= cos(u)" },
							
	{"35:Horn",  
							"x=  (a + u cos(w)) sin(b pi u)",
							"y=  (a + u cos(w)) cos(b pi u)\n    + c u",
							"z=  u sin(w)" },
												
	{"36:Pseudo Cross Cap",  
							"x=  (1 - u^2) sin(w)",
							"y=  (1 - u^2) sin(2 w)",
							"z=  u" },
							
	{"37:Cresent",  
							"x= (a + sin(b pi u) sin(b pi w))\n   sin(c pi w)",
							"y= (a + sin(b pi u) sin(b pi w))\n   cos(c pi w)",
							"z= cos(b pi u) sin(b pi w)\n   + 4 w - 2" },
							
	{"38:Apple Surface",  
							"x=  cos(u) (4 + 3.8 cos(w))",
							"y=  sin(u) (4 + 3.8 cos(w))",
							"z=  (cos(w)+sin(w)-1) (1+sin(w))\n    log(1-pi w/10)+7.5 sin(w)" },
	
	{"39:Trefoil Knot",  
							"x=  cos(u) cos(w) + 3 cos(u)\n    (1.5 + sin(1.5 u) / 2)",
							"y=  sin(u) cos(w) + 3 sin(u)\n    (1.5 + sin(1.5 u) / 2)",
							"z=  sin(w) + 2 cos(1.5 u)" },
	
	{"40:Torus",  
							"x=  (a + b cos(w)) cos(u)",
							"y=  (a + b cos(w)) sin(u)",
							"z=  b sin(w)" },
	
	{"41:Antisymmetric Torus",  
							"x=  (a + b cos(w) (c + sin(u)))\n    cos(u)",
							"y=  (a + b cos(w) (c + sin(u)))\n    sin(u)",
							"z=  b sin(w) (a + sin(u))" },
	
	{"42:Eight turn Torus",  
							"x=  (a + b (cos(u/2) sin(w)\n    - sin(u/2) sin(2 w))) cos(u)",
							"y=  (a + b (cos(u/2) sin(w)\n    - sin(u/2) sin(2 w))) sin(u)",
							"z=  b (sin(u/2) sin(w)\n    + cos(u/2) sin(2 w))" }};
	
//																								functions
int main() {	
	sys_slowOff(); hpg_set_mode_gray16(1); hpg_clear(); hpg_flip(); hpg_clear();
//																								compute surface
	surface_index = (int) sat_pop_real(); surface_index = abs(surface_index) % (MAX_FUNCTION + 1);
	compute_vertex();
	
    init_camera(&cam, &cam_pos, &cam_dir, 10.0, 50.0, 90.0, WINDOW_WIDTH, WINDOW_HEIGHT);
	surface.world_pos.x = 0; surface.world_pos.y = 0; surface.world_pos.z = 20; surface.world_pos.w = 1;

    while (!keyb_isON()) {	
//																								keyboard
		y_angle += 0.02;
		if (keyb_isAnyKeyPressed()) {		
//																								x y rotations & x y z translations		
			if( keyb_isUp() ) x_angle += 0.1; else if( keyb_isDown() ) x_angle += -0.1;
			else if( keyb_isLeft() ) y_angle += 0.1; else if( keyb_isRight() ) y_angle += -0.1;
			else if(keyb_isKeyPressed(0,5)) {if(surface.world_pos.z > 10.0)  surface.world_pos.z -= 1.0;}
			else if(keyb_isKeyPressed(0,4)) {if(surface.world_pos.z < 50.0)  surface.world_pos.z += 1.0;}
			else if(keyb_isKeyPressed(1,4)) {if(surface.world_pos.x < 20.0)  surface.world_pos.x += 1.0;} 
			else if(keyb_isKeyPressed(3,4)) {if(surface.world_pos.x > -20.0) surface.world_pos.x -= 1.0;} 
			else if(keyb_isKeyPressed(2,3)) {if(surface.world_pos.y < 20.0)  surface.world_pos.y += 1.0;}
			else if(keyb_isKeyPressed(2,5)) {if(surface.world_pos.y > -25.0) surface.world_pos.y -= 1.0;}
//																								z rotation & variables decrement 			
			else if (keyb_isLS()) { 
				if (keyb_isUp()) z_angle += 0.1; else if (keyb_isDown()) z_angle += -0.1;
				
				else if(keyb_isKeyPressed(1,1)) { surface_array[surface_index].u_min -= INC;compute_vertex();}
				else if(keyb_isKeyPressed(1,2)) { surface_array[surface_index].v_min -= INC;compute_vertex();}
				else if(keyb_isKeyPressed(0,1)) { surface_array[surface_index].u_max -= INC;compute_vertex();}
				else if(keyb_isKeyPressed(0,2)) { surface_array[surface_index].v_max -= INC;compute_vertex();}								
				
				else if(keyb_isKeyPressed(3,1)) { if(surface_array[surface_index].stepu > 2) surface_array[surface_index].stepu -= 1; compute_vertex();}
				else if(keyb_isKeyPressed(2,1)) { if(surface_array[surface_index].stepw > 2) surface_array[surface_index].stepw -= 1; compute_vertex();}
				
				else if(keyb_isKeyPressed(5,0)) { variable.a -= INC; compute_vertex();}
				else if(keyb_isKeyPressed(5,1)) { variable.b -= INC; compute_vertex();}
				else if(keyb_isKeyPressed(5,2)) { variable.c -= INC; compute_vertex();}
			}
//																								variables increment  u & w ranges  step & open	
			else if(keyb_isKeyPressed(1,1)) { surface_array[surface_index].u_min += INC;compute_vertex();}
			else if(keyb_isKeyPressed(1,2)) { surface_array[surface_index].v_min += INC;compute_vertex();}
			else if(keyb_isKeyPressed(0,1)) { surface_array[surface_index].u_max += INC;compute_vertex();}
			else if(keyb_isKeyPressed(0,2)) { surface_array[surface_index].v_max += INC;compute_vertex();}								
				
			else if(keyb_isKeyPressed(3,1)) { if(surface_array[surface_index].stepu < 25) surface_array[surface_index].stepu += 1; compute_vertex();}
			else if(keyb_isKeyPressed(2,1)) { if(surface_array[surface_index].stepw < 25) surface_array[surface_index].stepw += 1; compute_vertex();}
			
			else if(keyb_isKeyPressed(2,0)) { 
				surface_array[surface_index].open == 1 ? (surface_array[surface_index].open = 0) : (surface_array[surface_index].open = 1);
				compute_vertex();}
//																								
			else if(keyb_isKeyPressed(5,0)) { variable.a += INC; compute_vertex();}
			else if(keyb_isKeyPressed(5,1)) { variable.b += INC; compute_vertex();}
			else if(keyb_isKeyPressed(5,2)) { variable.c += INC; compute_vertex();}
//																								display variables 
			else if(keyb_isKeyPressed(5,6)) { display == 1 ? (display = 0) : (display = 1);}
//																								display surface info
			else if (keyb_isKeyPressed(6,5)) {
				hpg_clear(); 
				hpg_draw_text(surface_info[surface_index].name,65 - (strlen(surface_info[surface_index].name)*2) , 0);
				hpg_draw_text(surface_info[surface_index].x,0, 12);
				hpg_draw_text(surface_info[surface_index].y,0, 36);
				hpg_draw_text(surface_info[surface_index].z,0, 60);
				hpg_flip();keyb_getkey(1); 	
			}	
//																								freeze the screen
			else if(keyb_isKeyPressed(5,4)) keyb_getkey(1);

		}
//																								reload vertices
		for( vertex = 0; vertex < num_verts; vertex++) {			
			surface.vlist[vertex].x = vertex_array[vertex].x; surface.vlist[vertex].y = vertex_array[vertex].y;
			surface.vlist[vertex].z = vertex_array[vertex].z; surface.vlist[vertex].w = 1.0;
		}
//																								reset culling flag 
		for (polygone = 0; polygone < num_polys; polygone++) {surface.pplist[polygone]->cull = 0;}
		
		build_rotation_matrice(x_angle, y_angle, z_angle, &mrot);	
// 																								rotation in model space & translation into world space
// 																								then world coordinates to camera coordinates
		for (vertex = 0; vertex < num_verts; vertex++) {
			for (col = 0; col < 4; col++) {       						
				sum = 0; 
				for (row = 0; row < 4; row++) { sum += (surface.vlist[vertex].v.M[row] * mrot.M[row][col]);} 
				presult.M[col] = sum;
			} 
			surface.vlist[vertex].v.x = presult.x + surface.world_pos.x; surface.vlist[vertex].v.y = presult.y + surface.world_pos.y;
			surface.vlist[vertex].v.z = presult.z + surface.world_pos.z; surface.vlist[vertex].v.w = 1;	
			for (col = 0; col < 4; col++) {       						
				sum = 0; 
				for (row = 0; row < 4; row++) { sum += (surface.vlist[vertex].v.M[row] * cam.mcam.M[row][col]);} 
				presult.M[col] = sum;
			} 
			surface.vlist[vertex].v.x = presult.x; surface.vlist[vertex].v.y = presult.y;
			surface.vlist[vertex].v.z = presult.z; surface.vlist[vertex].v.w = presult.w;	
		}
// 																								sort polygones list on z average value 
		qsort((void *) surface.pplist, num_polys, sizeof(POLYGONE_PTR), compare_polygone);
//																								set culling flag on z value			
		for (polygone = 0; polygone < num_polys; polygone++) {
			if(surface.vlist[surface.pplist[polygone]->vert[0]].z <= EPSILON_E5 || surface.vlist[surface.pplist[polygone]->vert[1]].z <= EPSILON_E5
			|| surface.vlist[surface.pplist[polygone]->vert[2]].z <= EPSILON_E5 || surface.vlist[surface.pplist[polygone]->vert[3]].z <= EPSILON_E5
			) surface.pplist[polygone]->cull = 1;
		}
// 																								camera coordinates to screen coordinates
		float alpha = (0.5 * cam.viewport_width - 0.5); float beta = (0.5 * cam.viewport_height - 0.5);
		for (vertex = 0; vertex < num_verts; vertex++) {
			float z = surface.vlist[vertex].z; if(z == 0.0) z = EPSILON_E5;	
			surface.vlist[vertex].x = cam.view_dist * surface.vlist[vertex].x / z;
			surface.vlist[vertex].y = cam.view_dist * surface.vlist[vertex].y * cam.aspect_ratio / z;
			surface.vlist[vertex].x = alpha + alpha * surface.vlist[vertex].x;
			surface.vlist[vertex].y = beta - beta * surface.vlist[vertex].y;
		}
// 																								draw surface
		hpg_clear();
		for (polygone = 0; polygone < num_polys; polygone++) {
			if(surface.pplist[polygone]->cull == 1) continue;
			raster_x[0] = (int) surface.vlist[surface.pplist[polygone]->vert[0]].x;
			raster_y[0] = (int) surface.vlist[surface.pplist[polygone]->vert[0]].y;
			raster_x[1] = (int) surface.vlist[surface.pplist[polygone]->vert[1]].x;
			raster_y[1] = (int) surface.vlist[surface.pplist[polygone]->vert[1]].y;
			raster_x[2] = (int) surface.vlist[surface.pplist[polygone]->vert[2]].x;
			raster_y[2] = (int) surface.vlist[surface.pplist[polygone]->vert[2]].y;
			raster_x[3] = (int) surface.vlist[surface.pplist[polygone]->vert[3]].x;
			raster_y[3] = (int) surface.vlist[surface.pplist[polygone]->vert[3]].y;
			hpg_set_color(hpg_stdscreen,HPG_COLOR_GRAY_4); hpg_fill_polygon(raster_x,raster_y, 4);	
			hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); hpg_draw_polygon(raster_x, raster_y, 4);		
		}
		
		if(display) {
			sprintf(buffer, "umin...%.2f",  surface_array[surface_index].u_min); hpg_draw_text(buffer, 0, COL_U1);
			sprintf(buffer, "umax...%.2f",  surface_array[surface_index].u_max); hpg_draw_text(buffer, 0, COL_U2);
			sprintf(buffer, "wmin...%.2f",  surface_array[surface_index].v_min); hpg_draw_text(buffer, 0, COL_W3);
			sprintf(buffer, "wmax...%.2f",  surface_array[surface_index].v_max); hpg_draw_text(buffer, 0, COL_W4);
			sprintf(buffer, "stepu..%d",  surface_array[surface_index].stepu); hpg_draw_text(buffer, 0, COL_STEPU);
			sprintf(buffer, "stepw..%d",  surface_array[surface_index].stepw); hpg_draw_text(buffer, 0, COL_STEPW);
			sprintf(buffer, "open...%d",  surface_array[surface_index].open); hpg_draw_text(buffer, 0, COL_OPEN);
			if(variable.flag_a) {sprintf(buffer, "a..%.2f",  variable.a); hpg_draw_text(buffer, 0, COL_A);}
			if(variable.flag_b) {sprintf(buffer, "b..%.2f",  variable.b); hpg_draw_text(buffer, 0, COL_B);}
			if(variable.flag_c) {sprintf(buffer, "c..%.2f",  variable.c); hpg_draw_text(buffer, 0, COL_C);}
		}		
		hpg_flip();
	}
	sys_slowOn(); return 1;
}
	

void compute_vertex() {
	free( (void *) vertex_array) ;free( (void *) surface.vlist); free( (void *) surface.plist); free( (void *) surface.pplist);
	
	vertex_array = (VECTOR3D *)  malloc( (surface_array[surface_index].stepu + 1) * (surface_array[surface_index].stepw + 1) * sizeof(VECTOR3D));
	if (vertex_array == NULL) { hpg_clear(); hpg_draw_text("Error: insufficient memory!", 0, 0); hpg_flip(); keyb_getkey(1); return;}
	
	num_u = num_verts = 0; surface_array[surface_index].compute_surface(); 
	
	int num_v = num_verts / num_u; 
	if(!surface_array[surface_index].open) num_polys = (num_v - 1) * num_u; else num_polys = (num_v - 1) * (num_u - 1);
//																								reserve memory	
	surface.vlist  = (VERTEX_PTR)     malloc(num_verts * sizeof(VERTEX));
	surface.plist  = (POLYGONE_PTR)   malloc(num_polys * sizeof(POLYGONE));
	surface.pplist = (POLYGONE_PTR *) malloc(num_polys * sizeof(POLYGONE_PTR));
	if (surface.vlist == NULL || surface.plist == NULL || surface.pplist == NULL) { 
		hpg_clear(); hpg_draw_text("Error: insufficient memory!", 0, 0); hpg_flip(); keyb_getkey(1); return;
	}
//																								scale vertices
	for( vertex = 0; vertex < num_verts; vertex++) {
		vertex_array[vertex].x *= surface_array[surface_index].scale.x; 
		vertex_array[vertex].y *= surface_array[surface_index].scale.y;
		vertex_array[vertex].z *= surface_array[surface_index].scale.z; 
	}
//																								build polygones (quadrangles)
	int base = 0; polygone = 0; int nl;
	for(nl = 0; nl < num_u - 1; nl++) {															
		for(vertex = 0; vertex < num_v - 1; vertex++) {		
			surface.plist[polygone].vert[0] = vertex + base;
			surface.plist[polygone].vert[1] = vertex + 1 + base; 
			surface.plist[polygone].vert[2] = vertex + 1 + base + num_v;
			surface.plist[polygone].vert[3] = vertex + base + num_v;
			surface.pplist[polygone] = &surface.plist[polygone];
			polygone++;
		}
		base += num_v;
	}
//																								closed or opened	
	if(!surface_array[surface_index].open) 
		for(vertex = 0; vertex < num_v - 1; vertex++) {		
			surface.plist[polygone].vert[0] = vertex + base;
			surface.plist[polygone].vert[1] = vertex + 1 + base; 
			surface.plist[polygone].vert[2] = vertex + 1;
			surface.plist[polygone].vert[3] = vertex;
			surface.pplist[polygone] = &surface.plist[polygone];
			polygone++;
		}
}	
	
//																								surface functions
void compute_plane() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = u ;
			vertex_array[num_verts].y = 0.0;
			vertex_array[num_verts].z = v;			
			num_verts++;
		}
	}	
}

void compute_helicoid() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.0; variable.flag_a = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = u * cos(v);
			vertex_array[num_verts].y = u * sin(v);
			vertex_array[num_verts].z = variable.a * v * 2;			
			num_verts++;
		}
	}	
}

void compute_paraboloid() {	
	float u, v;
	if(variable.flag_a == 0 ) { variable.a = 5.0; variable.b = 7.0; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = variable.a * v * cos(u);
			vertex_array[num_verts].y = variable.a * v * sin(u);
			vertex_array[num_verts].z = variable.b * v*v;			
			num_verts++;
		}
	}	
}

void compute_funnel() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = u * cos(v);
			vertex_array[num_verts].y = u * sin(v);
			vertex_array[num_verts].z = log(u);
			num_verts++;
		}
	}	
}

void compute_tractroid() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = (exp(u) + exp(-u)) * 0.5;				//cosh(u)
			float f = (exp(u) - exp(-u)) * 0.5;				//sinh(u)
			vertex_array[num_verts].x = (1 / e) * cos(v);
			vertex_array[num_verts].y = (1 / e) * sin(v);
			vertex_array[num_verts].z = u - (f / e);
			num_verts++;
		}
	}	
}

void compute_hyperboloid() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.7; variable.b = 1.9; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = (exp(v) + exp(-v)) * 0.5;				//cosh(v)
			vertex_array[num_verts].x =  variable.a * e * cos(u);
			vertex_array[num_verts].y = variable.a * e * sin(u);
			vertex_array[num_verts].z = variable.b * ( (exp(v) - exp(-v)) * 0.5);
			num_verts++;
		}
	}	
}

void compute_swallow() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = u * v*v + 3 * v*v*v*v;
			vertex_array[num_verts].y = - 2 * u * v - 4 * v*v*v;
			vertex_array[num_verts].z = u;
			num_verts++;
		}
	}	
}

void compute_wallis() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.0; variable.b = 1.0; variable.c = 1.0; variable.flag_a = 1; variable.flag_b = 1; variable.flag_c = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = v * cos(u);
			vertex_array[num_verts].y = v * sin(u);
			vertex_array[num_verts].z = variable.c * sqrt( variable.a*variable.a - variable.b*variable.b * cos(u)*cos(u));
			num_verts++;
		}
	}	
}

void compute_spherical_wave() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = u * cos(cos(u));
			vertex_array[num_verts].x = e * cos(v);
			vertex_array[num_verts].y = e * sin(v);
			vertex_array[num_verts].z = u * sin(cos(u));
			num_verts++;
		}
	}	
}

void compute_bow_tie() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = sin(v) * cos(u);
			vertex_array[num_verts].y = 2.0 * cos(v);
			vertex_array[num_verts].z = 4.0 * sin(v) * cos(v);
			num_verts++;
		}
	}	
}

void compute_trash_can() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.0; variable.b = 1.0; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = (variable.b + v) * cos(u);
			vertex_array[num_verts].y = v * sin(u);
			vertex_array[num_verts].z = variable.a * v*v;
			num_verts++;
		}
	}	
}

void compute_spring() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.4; variable.b = 0.3; variable.c = 2.6; variable.flag_a = 1; variable.flag_b = 1; variable.flag_c = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = (variable.a + variable.b * cos(v)) * cos(u);
			vertex_array[num_verts].y = (variable.a + variable.b * cos(v)) * sin(u);
			vertex_array[num_verts].z =  variable.b * (sin(v) + variable.c * u / pi);
			num_verts++;
		}
	}	
}

void compute_cylinder() {	
	float u, v; if(variable.flag_a == 0 ) {variable.a = 1.0; variable.flag_a = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = variable.a * cos(u);
			vertex_array[num_verts].y = variable.a * sin(u);
			vertex_array[num_verts].z = v;
			num_verts++;
		}
	}	
}

void compute_disk() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.2; variable.b = 0.55; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {	
			vertex_array[num_verts].x = sin(u) / (variable.a + sin(v));
			vertex_array[num_verts].y = cos(v) / variable.b;
			vertex_array[num_verts].z = cos(u) / (variable.a + sin(v));
			num_verts++;
		}
	}	
}

void compute_wave_cosine() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.0; variable.b = 1.0; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {	
			vertex_array[num_verts].x = u;
			vertex_array[num_verts].y = variable.a * cos(variable.b * sqrt(u*u + v*v));
			vertex_array[num_verts].z = v;
			num_verts++;
		}
	}	
}

void compute_flying_saucer() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {	
			vertex_array[num_verts].x = cos(u) * cos(v);
			vertex_array[num_verts].y = sin(u) * sin(u) * sin(u);
			vertex_array[num_verts].z = cos(u) * sin(v);
			num_verts++;
		}
	}	
}

void compute_cornucopia() {
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.0; variable.b = 1.0; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = exp( variable.a * v);
			float f = exp( variable.b * v);
			vertex_array[num_verts].x = f * cos(v) + e * cos(u) * cos(v);
			vertex_array[num_verts].y = f * sin(v) + e * cos(u) * sin(v);
			vertex_array[num_verts].z = e * sin(u);
			num_verts++;
		}
	}	
}

void compute_enneper() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {													
			vertex_array[num_verts].x = u - (u*u*u  * 0.33333) + (u * v*v);	
			vertex_array[num_verts].y = v - (v*v*v  * 0.33333) + (u*u * v);
			vertex_array[num_verts].z = u*u - v*v;
			num_verts++;
		}
	}	
}

void compute_handkerchief() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {													
			vertex_array[num_verts].x = u;	
			vertex_array[num_verts].y = v;
			vertex_array[num_verts].z = (u*u*u * 0.33333) + (u * v*v) + (2 * (u*u - v*v));
			num_verts++;
		}
	}	
}
 
void compute_klein_bottle() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 7.0; variable.b = 17.5; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		float e = 4.0 * (1.0 - cos(u) * 0.5);
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {		
			if( u >= 0.0 && u < pi ) {
				vertex_array[num_verts].x = variable.a * cos(u) * (1 + sin(u)) + (e * cos(u) * cos(v));	
				vertex_array[num_verts].y = variable.b * sin(u) + (e * sin(u) * cos(v));
				vertex_array[num_verts].z = e * sin(v);
			}
			else {
				vertex_array[num_verts].x = variable.a * cos(u) * (1 + sin(u)) + (e * cos(v + pi));
				vertex_array[num_verts].y = variable.b * sin(u);
				vertex_array[num_verts].z = e * sin(v);
			}
			num_verts++;
		}
	}	
}

void compute_monkey() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {												
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {													
			vertex_array[num_verts].x = u;	
			vertex_array[num_verts].y = v;
			vertex_array[num_verts].z = u*u*u - 3.0 * u * v*v;
			num_verts++;
		}
	}	
}
 
void compute_plucker() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {													
			vertex_array[num_verts].x = u * sqrt(1.0 - (v*v));
			vertex_array[num_verts].y = u * v;
			vertex_array[num_verts].z = 1.0 - (v*v);			
			num_verts++;
		}
	}	
} 
 
void compute_roman() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {													
			float e = sqrt(1 - u*u);
			vertex_array[num_verts].x = 2.0 * u * cos(v) * e;
			vertex_array[num_verts].y = 2.0 * u * sin(v) * e;
			vertex_array[num_verts].z = 1.0 - 2.0 * u*u * cos(v) * cos(v);			
			num_verts++;
		}
	}	
}

void compute_shoe() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = u;
			vertex_array[num_verts].y = v;
			vertex_array[num_verts].z = (u*u*u * 0.33333) - (v*v * 0.5);			
			num_verts++;
		}
	}	
}

void compute_screw() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = u * cos(v);
			vertex_array[num_verts].y = u * sin(v);	
			vertex_array[num_verts].z = v * cos(u);			
			num_verts++;
		}
	}	
}

void compute_whitney() {	
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = u * v;
			vertex_array[num_verts].y = u ;
			vertex_array[num_verts].z = v*v;			
			num_verts++;
		}
	}	
}

void compute_catenoid() {	
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 0.8; variable.flag_a = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = (exp(v) + exp(-v)) * 0.5;				//cosh(v)
			vertex_array[num_verts].x = variable.a * e * cos(u);
			vertex_array[num_verts].y = variable.a * e * sin(u) ;
			vertex_array[num_verts].z = v;			
			num_verts++;
		}
	}	
}

void compute_snail() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {	
			vertex_array[num_verts].x = u * cos(v) * sin(u);
			vertex_array[num_verts].y = u * cos(u) * cos(v);
			vertex_array[num_verts].z = -u * sin(v);
			num_verts++;
		}
	}	
}

void compute_cosine() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {	
			vertex_array[num_verts].x = cos(u);
			vertex_array[num_verts].y = cos(v);
			vertex_array[num_verts].z = cos(u + v);
			num_verts++;
		}
	}	
}

void compute_ellipsoid() {
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.0; variable.b = 1.0; variable.c = 1.0; variable.flag_a = 1; variable.flag_b = 1; variable.flag_c = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {	
			vertex_array[num_verts].x = variable.a * cos(v) * sin(u);
			vertex_array[num_verts].y = variable.b * sin(u) * sin(v);
			vertex_array[num_verts].z = variable.c * cos(u);
			num_verts++;
		}
	}	
}

void compute_eight() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {	
			vertex_array[num_verts].x = cos(u) * sin(2 * v);
			vertex_array[num_verts].y = sin(u) * sin(2 * v);
			vertex_array[num_verts].z = sin(v);
			num_verts++;
		}
	}	
}

void compute_kidney() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {	
			vertex_array[num_verts].x = cos(u) * (3 * cos(v) - cos(3 * v));
			vertex_array[num_verts].y = sin(u) * (3 * cos(v) - cos(3 * v));
			vertex_array[num_verts].z = 3 * sin(v) - sin(3 * v);
			num_verts++;
		}
	}	
}

void compute_drop() {
	float u, v;
	if(variable.flag_a == 0 ) {variable.a = 1.0; variable.b = 1.0; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = variable.a * (variable.b - cos(u)) * sin(u);
			vertex_array[num_verts].x = e * cos(v);
			vertex_array[num_verts].y = e * sin(v);
			vertex_array[num_verts].z = cos(u);
			num_verts++;
		}
	}	
}

void compute_pillow() {
	float u, v;
		if(variable.flag_a == 0) {variable.a = 0.5; variable.flag_a = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = cos(u);
			vertex_array[num_verts].y = cos(v);
			vertex_array[num_verts].z = variable.a * sin(u) * sin(v);
			num_verts++;
		}
	}	
}

void compute_fish() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = (cos(u) - cos(2 * u)) * cos(v) * 0.25;
			vertex_array[num_verts].y = (sin(u) - sin(2 * u)) * sin(v) * 0.25;
			vertex_array[num_verts].z =	cos(u);
			num_verts++;
		}
	}	
}

void compute_horn() {
	float u, v;
	if(variable.flag_a == 0) {variable.a = 2.0; variable.b = 2.0; variable.c = 2.0; variable.flag_a = 1; variable.flag_b = 1; variable.flag_c = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = variable.a + u * cos(v);
			float f = variable.b * pi * u;
			vertex_array[num_verts].x = e * sin(f);
			vertex_array[num_verts].y = e * cos(f) + variable.c * u;
			vertex_array[num_verts].z = u * sin(v);
			num_verts++;
		}
	}	
}

void compute_pseudo_cross_cap() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = (1.0 - u*u);
			vertex_array[num_verts].x = e * sin(v);
			vertex_array[num_verts].y = e * sin(2.0 * v);
			vertex_array[num_verts].z = u;
			num_verts++;
		}
	}	
}

void compute_cresent() {
	float u, v;
	if(variable.flag_a == 0) {variable.a = 2.0; variable.b = 2.2; variable.c = 3.0;variable.flag_a = 1; variable.flag_b = 1; variable.flag_c = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = (variable.a + sin(variable.b * pi * u) * sin(variable.b * pi * v));
			float f = variable.c * pi * v;
			vertex_array[num_verts].x = e * sin(f);
			vertex_array[num_verts].y = e * cos(f);
			vertex_array[num_verts].z = cos(variable.b * pi * u) * sin(variable.b * pi * v) + 4 * v - 2.0;
			num_verts++;
		}
	}	
}

void compute_apple() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = 4.0 + 3.8 * cos(v);
			vertex_array[num_verts].x =  cos(u) * e;
			vertex_array[num_verts].y =  sin(u) * e;
			vertex_array[num_verts].z = (cos(v) + sin(v) - 1.0) * (1.0 + sin(v)) * log(1.0 - pi * v * 0.1) + 7.5 * sin(v);
			num_verts++;
		}
	}	
}

void compute_trefoilknot() {
	float u, v;
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = cos(u) * cos(v) + 3.0 * cos(u) * (1.5 + sin(1.5 * u) * 0.5);
			vertex_array[num_verts].y = sin(u) * cos(v) + 3.0 * sin(u) * (1.5 + sin(1.5 * u) * 0.5);
			vertex_array[num_verts].z = sin(v) + 2 * cos(1.5 * u);
			num_verts++;
		}
	}	
}	

void compute_torus() {
	float u, v;
	if(variable.flag_a == 0) {variable.a = 3.0; variable.b = 1.0; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			vertex_array[num_verts].x = cos(u) * (variable.a + variable.b * cos(v));
			vertex_array[num_verts].y = sin(u) * (variable.a + variable.b * cos(v));
			vertex_array[num_verts].z = sin(v);
			num_verts++;
		}
	}	
}			

void compute_antisymetric_torus() {
	float u, v;
	if(variable.flag_a == 0) {variable.a = 3.0; variable.b = 1.0; variable.c = 1.0; variable.flag_a = 1; variable.flag_b = 1; variable.flag_c = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = variable.a + variable.b * cos(v) * (variable.c + sin(u));
			vertex_array[num_verts].x = e * cos(u);
			vertex_array[num_verts].y = e * sin(u);
			vertex_array[num_verts].z = variable.b * sin(v) * (variable.c + sin(u));
			num_verts++;
		}
	}	
}		

void compute_turn_torus() {
	float u, v;
	if(variable.flag_a == 0) {variable.a = 3.0; variable.b = 1.0; variable.flag_a = 1; variable.flag_b = 1;}
	float step_u = (surface_array[surface_index].u_max - surface_array[surface_index].u_min) / (float) surface_array[surface_index].stepu;
	float step_v = (surface_array[surface_index].v_max - surface_array[surface_index].v_min) / (float) surface_array[surface_index].stepw;
	for(u = surface_array[surface_index].u_min; u < surface_array[surface_index].u_max; u += step_u) {															
		num_u++;
		for(v = surface_array[surface_index].v_min; v < surface_array[surface_index].v_max; v += step_v) {
			float e = variable.a + variable.b * (cos(u * 0.5) * sin(v) - sin(u * 0.5) * sin(2.0 * v));
			vertex_array[num_verts].x = e * cos(u);
			vertex_array[num_verts].y = e * sin(u);
			vertex_array[num_verts].z = variable.b * (sin(u * 0.5) * sin(v) + cos(u * 0.5) * sin(2.0 * v));
			num_verts++;
		}
	}	
}		

//																								routines
int compare_polygone(const void *arg1, const void *arg2) {
	float z1, z2; POLYGONE_PTR poly_1, poly_2;
	poly_1 = *((POLYGONE_PTR *)(arg1)); poly_2 = *((POLYGONE_PTR *)(arg2));
	z1 = (float)0.25*(surface.vlist[poly_1->vert[0]].z + surface.vlist[poly_1->vert[1]].z 
		+ surface.vlist[poly_1->vert[2]].z + surface.vlist[poly_1->vert[3]].z);
	z2 = (float)0.25*(surface.vlist[poly_2->vert[0]].z + surface.vlist[poly_2->vert[1]].z 
		+ surface.vlist[poly_2->vert[2]].z + surface.vlist[poly_2->vert[3]].z);
	if (z1 > z2) return(-1); else if (z1 < z2) return(1); else return(0);
} 

void init_plane(PLANE_PTR plane, POINT3D_PTR p0, VECTOR3D_PTR normal) {
	plane->p0.x = p0->x;plane->p0.y = p0->y;plane->p0.z = p0->z;								
	float length = (float) sqrt( normal->x * normal->x + normal->y * normal->y + normal->z * normal->z);
	if (length < EPSILON_E5) return;							
	plane->n.x = normal->x / length; plane->n.y = normal->y / length; plane->n.z = normal->z / length;
} 

void init_matrice(MATRICE_PTR ma,	float m00, float m01, float m02, float m03, float m10, float m11, float m12, float m13,
										float m20, float m21, float m22, float m23, float m30, float m31, float m32, float m33) {
	ma->M00 = m00; ma->M01 = m01; ma->M02 = m02; ma->M03 = m03;
	ma->M10 = m10; ma->M11 = m11; ma->M12 = m12; ma->M13 = m13;
	ma->M20 = m20; ma->M21 = m21; ma->M22 = m22; ma->M23 = m23;
	ma->M30 = m30; ma->M31 = m31; ma->M32 = m32; ma->M33 = m33;
} 

void multiply_matrice(MATRICE_PTR ma, MATRICE_PTR mb, MATRICE_PTR mprod) {
	int row,col,index;
	for (row=0; row<4; row++) {
		for (col=0; col<4; col++) {
			float sum = 0; 
			for (index = 0; index < 4; index++) {
				sum += (ma->M[row][index] * mb->M[index][col]);
			} 
			mprod->M[row][col] = sum;
        } 
    } 
} 

void build_rotation_matrice(float theta_x, float theta_y, float theta_z, MATRICE_PTR mrot)  {
	MATRICE mx, my, mz, mtmp;       							
	float sin_theta = 0, cos_theta = 0;						   	  							
	MAT_IDENTITY(mrot);										
	cos_theta = cos(theta_x);
	sin_theta = sin(theta_x);
	init_matrice(&mx, 	1,    0,         0,         0,
						0,    cos_theta, sin_theta, 0,
						0,   -sin_theta, cos_theta, 0,
						0,    0,         0,         1);
	cos_theta = cos(theta_y);
	sin_theta = sin(theta_y);
	init_matrice(&my,	cos_theta, 0, -sin_theta, 0,  
						0,         1,  0,         0,
						sin_theta, 0,  cos_theta,  0,
						0,         0,  0,          1);
	cos_theta = cos(theta_z);
	sin_theta = sin(theta_z);
	init_matrice(&mz, 	cos_theta, 	sin_theta, 0, 0,  
						-sin_theta, cos_theta, 0, 0,
						0,         	0,         1, 0,
						0,         	0,         0, 1);
	multiply_matrice(&mx, &my, &mtmp);
	multiply_matrice(&mtmp, &mz, mrot);
}

void init_camera(CAMERA_PTR cam, POINT4D_PTR cam_pos, VECTOR4D_PTR cam_dir, float near_z, float far_z, float fov, float viewport_width, float viewport_height) {
		cam->pos.x = cam_pos->x;cam->pos.y = cam_pos->y;cam->pos.z = cam_pos->z;cam->pos.w = cam_pos->w;
		cam->dir.x = cam_dir->x;cam->dir.y = cam_dir->y;cam->dir.z = cam_dir->z;cam->dir.w = cam_dir->w;
		cam->near_z = near_z; cam->far_z  = far_z;     	
		cam->viewport_width  = viewport_width; cam->viewport_height = viewport_height;
		cam->viewport_center_x = (viewport_width-1)/2; cam->viewport_center_y = (viewport_height-1)/2;
		cam->aspect_ratio = (float) viewport_width / (float) viewport_height;
		MAT_IDENTITY(&cam->mcam); MAT_IDENTITY(&cam->mper); MAT_IDENTITY(&cam->mscr);
		cam->fov = fov; cam->viewplane_width  = 2.0; cam->viewplane_height = 2.0/cam->aspect_ratio;
		float tan_fov_div2 = tan(DEG_TO_RAD(fov / 2));
		cam->view_dist = (0.5) * (cam->viewplane_width) * tan_fov_div2;
		POINT3D pt_origin; pt_origin.x = 0.0; pt_origin.y = 0.0; pt_origin.z = 0.0;
		VECTOR3D vn; 
		vn.x =  1.0;vn.y =  0.0;vn.z = -1.0; init_plane(&cam->rt_plane, &pt_origin, &vn);
		vn.x = -1.0;vn.y =  0.0;vn.z = -1.0; init_plane(&cam->lt_plane, &pt_origin, &vn);
		vn.x =  0.0;vn.y =  1.0;vn.z = -1.0; init_plane(&cam->tp_plane, &pt_origin, &vn);
		vn.x =  0.0;vn.y = -1.0;vn.z = -1.0; init_plane(&cam->bt_plane, &pt_origin, &vn); 		
}
























